/**
 * Endpoint for sign pass create
 * @class SignPassCreateEndpoint
 */
'use strict'

const { CreateEndpoint } = require('apeman-app-rest/lib/endpoints')
const validateDataType = require('apeman-app-rest/lib/validating/validate_data_type')
const schemas = require('../schemas')
const errors = require('../errors')
const co = require('co')
const uuid = require('uuid')
const { toData, toType } = require('apemanmodel')
const { ApSignPassModel } = require('../models')

/** @lends SignPassCreateEndpoint */
class SignPassCreateEndpoint extends CreateEndpoint {
  /**
   * @override
   */
  getDefaultConfig () {
    return {
      name: 'signPassCreate',
      schema: schemas('sign-pass-create')
    }
  }

  /**
   * @override
   */
  handle (ctx, next) {
    const s = this
    let succeed = s.bindSucceed(ctx)
    let fail = s.bindFail(ctx)
    return co(function * create () {
      let { state } = ctx
      let { model, data } = state
      let error = s.validate(data, state)
      if (error) {
        return fail(error)
      }
      let sign = yield s.sign(data, state)
      if (!sign) {
        return fail(errors.signError())
      }
      let type = toType(ApSignPassModel)
      let id = uuid.v4()
      let pass = {
        type, id,
        relationships: {
          sign: {
            data: toData.withoutAttributes(model, sign)
          }
        }
      }
      ctx.signed({ pass, sign })
      return succeed(
        {},
        pass,
        {},
        [
          toData(model, sign)
        ]
      )
    })
  }

  validate (data, state) {
    let type = toType(ApSignPassModel)
    let typeError = validateDataType(data, type)
    if (typeError) {
      return typeError
    }
    return null
  }

  sign (data, state) {
    const s = this
    let { model, transaction, digest } = state
    let { key, password } = data.attributes
    return co(function * () {
      let sign = yield model.findOne({
        transaction,
        where: { key },
        include: [
          model.associations[ 'user' ]
        ]
      })

      let hash = yield digest(password, sign.hashSalt)
      if (!hash === sign.hash) {
        return null
      }
      return sign
    })
  }
}

module.exports = SignPassCreateEndpoint
