/**
 * Test case for create.
 * Runs with mocha.
 */
'use strict'

const assert = require('assert')
const create = require('../lib/create.js')
const apemanport = require('apemanport')
const fs = require('fs')
const apemandb = require('apemandb')
const apemanmodel = require('apemanmodel')
const apemanApp = require('apeman-app')
const apemanrequest = require('apemanrequest')
const co = require('co')
const apemansleep = require('apemansleep')
let tmpDir = `${__dirname}/../tmp`
let dbFile = tmpDir + '/testing-db01.db'

const loc = require('../doc/mocks/locales')

describe('create', () => {
  let db, baseUrl, server
  let request = apemanrequest.create({ jar: true })
  let sleep = apemansleep.create()
  let user01
  let sign01, sign02
  let pass01

  before(() => co(function * () {
    db = apemandb({
      env: {
        DATABASE: 'testing-database02',
        USERNAME: 'FOO',
        PASSWORD: 'BAR',
        DIALECT: 'sqlite',
        STORAGE: dbFile,
        LOGGING: false
      },
      models: `${apemanmodel.paths.SAMPLE_DEFS_DIR}/*.json`
    })
    yield db.sync({ force: true })
  }))
  before(() => co(function * () {
    const { User, Session, UserSign } = db.models
    let port = yield apemanport.find()
    server = yield apemanApp('testing-app', {
      port,
      configuration: {
        $apps: {
          'testing-app': {
            '/': [
              require('apeman-app-body')(),
              require('apeman-app-link')({
                protocol: 'http'
              }),
              require('apeman-app-locale')(
                loc
              ),
              require('apeman-app-jsonapi')(),
              require('apeman-app-session')(Session),
              require('apeman-app-signed')()
            ],
            '/api': [
              require('apeman-app-rest')(User, {
                pathname: '/users'
              }),
              create(UserSign, {
                pathname: '/sign',
                passwordSpec: 'pdkdf2:120:184'
              })
            ]
          }
        }
      }
    })
    baseUrl = `http://localhost:${port}`
  }))

  after(() => co(function * () {
    yield sleep.sleep(100)
    fs.unlinkSync(dbFile)
  }))

  after(() => co(function * () {
    yield server.close()
  }))

  it('Endpoint spec', () => co(function * () {
    let res = yield request({
      method: 'OPTIONS',
      url: `${baseUrl}/api/users/`,
      json: true
    })
    let { body, statusCode } = res
    assert.equal(statusCode, 200)
    assert.ok(body)
  }))
  it('Create user', () => co(function * () {
    let res = yield request({
      method: 'POST',
      url: `${baseUrl}/api/users/`,
      json: true,
      body: {
        data: {
          type: 'users',
          attributes: {
            key: 'foo_bar',
            email: 'apbc@example.com'
          }
        }
      }
    })
    let { body, statusCode } = res
    assert.equal(statusCode, 201)
    let { data } = body
    user01 = Object.assign(
      {},
      data.attributes,
      { id: data.id }
    )
    assert.equal(user01.key, 'foo_bar')
    assert.equal(user01.email, 'apbc@example.com')
  }))

  it('Create sign', () => co(function * () {
    let res = yield request({
      method: 'POST',
      url: `${baseUrl}/api/sign`,
      json: true,
      body: {
        data: {
          type: 'user_signs',
          attributes: {
            key: user01.key,
            password: 'the-password-of-user01',
            kind: 'user.key'
          },
          relationships: {
            user: {
              data: {
                type: 'users',
                id: user01.id
              }
            }
          }
        }
      }
    })
    let { body, statusCode } = res
    assert.equal(statusCode, 201)
    let { data } = body
    assert.ok(data.attributes.signupAt)
    console.log(data.attributes)
    assert.ok(!data.attributes.hash)
    assert.ok(!data.attributes.hashSalt)
    sign01 = Object.assign({ id: data.id }, data.attributes)
  }))

  // it('Bulk create sign', () => co(function * () {
  //   let res = yield request({
  //     method: 'POST',
  //     url: `${baseUrl}/api/sign`,
  //     json: true,
  //     body: {
  //       data: [
  //         {
  //           type: 'user_signs',
  //           attributes: {
  //             key: user01.email,
  //             password: 'the-password-of-user01',
  //             kind: 'user.email'
  //           }
  //         }
  //       ]
  //
  //     }
  //   })
  //   let { body, statusCode } = res
  //   assert.equal(statusCode, 201)
  //   let { data } = body
  //   assert.ok(data[ 0 ].attributes.signupAt)
  //   assert.ok(!data[ 0 ].attributes.hash)
  //   assert.ok(!data[ 0 ].attributes.hashSalt)
  //   sign02 = Object.assign({ id: data[ 0 ].id }, data[ 0 ].attributes)
  // }))

  it('Update sign', () => co(function * () {
    let res = yield request({
      method: 'PATCH',
      url: `${baseUrl}/api/sign/${sign01.id}`,
      json: true,
      body: {
        data: {
          type: 'user_signs',
          id: sign01.id,
          attributes: {
            key: user01.key,
            password: 'the-password-of-user01-updated',
            kind: 'user.key'
          }
        }
      }
    })
    let { body, statusCode } = res
    assert.equal(statusCode, 200)
    let { data } = body
    assert.ok(data.attributes.signmodAt)
  }))

  // it('Bulk update signs', () => co(function * () {
  //   let res = yield request({
  //     method: 'PATCH',
  //     url: `${baseUrl}/api/sign`,
  //     json: true,
  //     body: {
  //       data: [
  //         {
  //           type: 'user_signs',
  //           id: sign01.id,
  //           attributes: {
  //             key: user01.key,
  //             password: 'the-password-of-user01-updated-bulk',
  //             kind: 'user.key'
  //           }
  //         }
  //       ]
  //     }
  //   })
  //   let { body, statusCode } = res
  //   assert.equal(statusCode, 200)
  //   let { data } = body
  //   let data0 = data[ 0 ]
  //   assert.ok(data0.attributes.signmodAt)
  // }))

  it('Create sign pass', () => co(function * () {
    let res = yield request({
      method: 'POST',
      url: `${baseUrl}/api/sign/pass`,
      json: true,
      body: {
        data: {
          type: 'user_sign_passes',
          attributes: {
            key: sign01.key,
            password: 'the-password-of-user01-updated-bulk'
          }
        }
      }
    })
    let { body, headers, statusCode } = res
    assert.equal(statusCode, 200)
    let { data } = body
    assert.equal(data.type, 'user_sign_passes')
    assert.ok(data.relationships)
    assert.ok(data.relationships.sign)
    assert.ok(data.relationships.sign.data)
    pass01 = Object.assign({ id: data.id }, data.attributes)
  }))

  it('Create sign pass', () => co(function * () {
    let res = yield request({
      method: 'POST',
      url: `${baseUrl}/api/sign/pass`,
      json: true,
      body: {
        data: {
          type: 'user_sign_passes',
          attributes: {
            key: sign01.key,
            password: 'the-password-of-user01-updated-bulk'
          }
        }
      }
    })
    let { body } = res
    let { data } = body
    assert.equal(data.type, 'user_sign_passes')
    assert.ok(data.relationships)
    assert.ok(data.relationships.sign)
    assert.ok(data.relationships.sign.data)
    pass01 = Object.assign({ id: data.id }, data.attributes)
  }))

  it('Get sign pass', () => co(function * () {
    let res = yield request({
      method: 'GET',
      url: `${baseUrl}/api/sign/pass/singleton`,
      json: true
    })
    let { body } = res
    assert.equal(res.statusCode, 200)
    let { data, included } = body
    assert.equal(data.type, 'user_sign_passes')
    assert.ok(data.relationships.sign.data)
    included.forEach((included) => {
      let attributes = included.attributes
      assert.ok(!attributes.hash)
      assert.ok(!attributes.hashSalt)
    })
  }))


  it('Destroy sign', () => co(function * () {
    let res = yield request({
      method: 'DELETE',
      url: `${baseUrl}/api/sign/${sign01.id}`,
      json: true
    })
    let { body, statusCode } = res
    assert.equal(statusCode, 200)
    assert.equal(body.meta.action, 'signDestroy')
    assert.equal(body.meta.count, 1)
  }))

  // it('Bulk destroy sign', () => co(function * () {
  //   let res = yield request({
  //     method: 'DELETE',
  //     url: `${baseUrl}/api/sign`,
  //     json: true,
  //     body: {
  //       data: [ {
  //         type: 'user_signs',
  //         id: sign02.id
  //       } ]
  //     }
  //   })
  //   let { body, statusCode } = res
  //   assert.equal(statusCode, 200)
  //   assert.equal(body.meta.action, 'bulkDestroy')
  //   assert.equal(body.meta.count, 1)
  // }))
})

/* global describe, before, after, it */
